@@ -26,12 +26,14 @@ __all__ = [
     'BooleanField',
     'CharField',
     'Clause',
+    'CompositeKey',
     'DateField',
     'DateTimeField',
     'DecimalField',
     'DoesNotExist',
     'DoubleField',
     'DQ',
+    'Entity',
     'Field',
     'FloatField',
     'fn',
@@ -170,8 +172,7 @@ JOIN_LEFT_OUTER = 'left outer'
 JOIN_FULL = 'full'
 
 def dict_update(orig, extra):
-    new = {}
-    new.update(orig)
+    new = orig.copy()
     new.update(extra)
     return new
 
@@ -318,6 +319,17 @@ class Clause(Node):
     def clone_base(self):
         return Clause(*self.nodes)
 
+class Entity(Node):
+    def __init__(self, *path):
+        super(Entity, self).__init__()
+        self.path = path
+
+    def clone_base(self):
+        return Entity(*self.path)
+
+    def __getattr__(self, attr):
+        return Entity(*self.path + (attr,))
+
 Join = namedtuple('Join', ('model_class', 'join_type', 'on'))
 
 class FieldDescriptor(object):
@@ -338,6 +350,7 @@ class Field(Node):
     _order = 0
     db_field = 'unknown'
     template = '%(column_type)s'
+    template_extra = ''
 
     def __init__(self, null=False, index=False, unique=False,
                  verbose_name=None, help_text=None, db_column=None,
@@ -702,6 +715,25 @@ class ForeignKeyField(IntegerField):
         return self.rel_model._meta.primary_key.db_value(value)
 
 
+class CompositeKey(object):
+    sequence = None
+
+    def __init__(self, *fields):
+        self.fields = fields
+
+    def add_to_class(self, model_class, name):
+        self.name = name
+        setattr(model_class, name, self)
+
+    def __get__(self, instance, instance_type=None):
+        if instance is not None:
+            return [getattr(instance, field) for field in self.fields]
+        return self
+
+    def __set__(self, instance, value):
+        pass
+
+
 class QueryCompiler(object):
     field_map = {
         'bigint': 'BIGINT',
@@ -817,6 +849,9 @@ class QueryCompiler(object):
         elif isinstance(node, Model):
             sql = self.interpolation
             params = [node.get_id()]
+        elif isinstance(node, Entity):
+            sql = '.'.join(map(self.quote, node.path))
+            params = []
         elif isclass(node) and issubclass(node, Model):
             sql = self.quote(node._meta.db_table)
             params = []
@@ -871,15 +906,16 @@ class QueryCompiler(object):
         return '', []
 
     def calculate_alias_map(self, query, start=1):
-        alias_map = {query.model_class: 't%s' % start}
+        make_alias = lambda model: model._meta.table_alias or 't%s' % start
+        alias_map = {query.model_class: make_alias(query.model_class)}
         for model, joins in query._joins.items():
             if model not in alias_map:
                 start += 1
-                alias_map[model] = 't%s' % start
+                alias_map[model] = make_alias(model)
             for join in joins:
                 if join.model_class not in alias_map:
                     start += 1
-                    alias_map[join.model_class] = 't%s' % start
+                    alias_map[join.model_class] = make_alias(join.model_class)
         return alias_map
 
     def generate_joins(self, joins, model_class, alias_map):
@@ -973,8 +1009,11 @@ class QueryCompiler(object):
             parts.append('LIMIT %s' % limit)
         if query._offset:
             parts.append('OFFSET %s' % query._offset)
-        if query._for_update:
+        for_update, no_wait = query._for_update
+        if for_update:
             parts.append('FOR UPDATE')
+            if no_wait:
+                parts.append('NOWAIT')
 
         return ' '.join(parts), params
 
@@ -1033,11 +1072,12 @@ class QueryCompiler(object):
             parts.append('NOT NULL')
         if field.primary_key:
             parts.append('PRIMARY KEY')
+        if field.template_extra:
+            parts.append(field.template_extra)
         if isinstance(field, ForeignKeyField):
             ref_mc = (
                 self.quote(field.rel_model._meta.db_table),
-                self.quote(field.rel_model._meta.primary_key.db_column),
-            )
+                self.quote(field.rel_model._meta.primary_key.db_column))
             parts.append('REFERENCES %s (%s)' % ref_mc)
             parts.append('%(cascade)s%(extra)s')
         elif field.sequence:
@@ -1050,8 +1090,12 @@ class QueryCompiler(object):
             parts.append('IF NOT EXISTS')
         meta = model_class._meta
         parts.append(self.quote(meta.db_table))
-        columns = ', '.join(self.field_sql(f) for f in meta.get_fields())
-        parts.append('(%s)' % columns)
+        columns = map(self.field_sql, meta.get_fields())
+        if isinstance(meta.primary_key, CompositeKey):
+            pk_cols = map(self.quote, (
+                meta.fields[f].db_column for f in meta.primary_key.fields))
+            columns.append('PRIMARY KEY (%s)' % ', '.join(pk_cols))
+        parts.append('(%s)' % ', '.join(columns))
         return parts
 
     def create_table(self, model_class, safe=False):
@@ -1489,7 +1533,7 @@ class SelectQuery(Query):
         self._limit = None
         self._offset = None
         self._distinct = False
-        self._for_update = False
+        self._for_update = (False, False)
         self._naive = False
         self._tuples = False
         self._dicts = False
@@ -1561,8 +1605,8 @@ class SelectQuery(Query):
         self._distinct = is_distinct
 
     @returns_clone
-    def for_update(self, for_update=True):
-        self._for_update = for_update
+    def for_update(self, for_update=True, nowait=False):
+        self._for_update = (for_update, nowait)
 
     @returns_clone
     def naive(self, naive=True):
@@ -1676,9 +1720,12 @@ class SelectQuery(Query):
         start = end = None
         res = self.execute()
         if isinstance(value, slice):
-            res.fill_cache(value.stop)
+            index = value.stop
         else:
-            res.fill_cache(value)
+            index = value
+        if index >= 0:
+            index += 1
+        res.fill_cache(index)
         return res._result_cache[value]
 
 class UpdateQuery(Query):
@@ -1813,6 +1860,9 @@ class Database(object):
     def rows_affected(self, cursor):
         return cursor.rowcount
 
+    def sql_error_handler(self, exception, sql, params, require_commit):
+        raise exception
+
     def compiler(self):
         return self.compiler_class(
             self.quote_char, self.interpolation, self.field_overrides,
@@ -1821,7 +1871,11 @@ class Database(object):
     def execute_sql(self, sql, params=None, require_commit=True):
         logger.debug((sql, params))
         cursor = self.get_cursor()
-        res = cursor.execute(sql, params or ())
+        try:
+            res = cursor.execute(sql, params or ())
+        except Exception as exc:
+            logger.error('Error executing query %s (%s)' % (sql, params))
+            return self.sql_error_handler(exc, sql, params, require_commit)
         if require_commit and self.get_autocommit():
             self.commit()
         return cursor
@@ -1913,6 +1967,8 @@ class SqliteDatabase(Database):
         OP_LIKE: 'GLOB',
         OP_ILIKE: 'LIKE',
     }
+    if sqlite3:
+        ConnectionError = sqlite3.OperationalError
 
     def _connect(self, database, **kwargs):
         if not sqlite3:
@@ -2004,7 +2060,7 @@ class PostgresqlDatabase(Database):
 class MySQLDatabase(Database):
     commit_select = True
     field_overrides = {
-        'boolean': 'BOOL',
+        'bool': 'BOOL',
         'decimal': 'NUMERIC',
         'double': 'DOUBLE PRECISION',
         'float': 'FLOAT',
@@ -2130,7 +2186,7 @@ default_database = SqliteDatabase('peewee.db')
 
 class ModelOptions(object):
     def __init__(self, cls, database=None, db_table=None, indexes=None,
-                 order_by=None, primary_key=None):
+                 order_by=None, primary_key=None, table_alias=None, **kwargs):
         self.model_class = cls
         self.name = cls.__name__.lower()
         self.fields = {}
@@ -2139,14 +2195,19 @@ class ModelOptions(object):
 
         self.database = database or default_database
         self.db_table = db_table
-        self.indexes = indexes or []
+        self.indexes = list(indexes or [])
         self.order_by = order_by
         self.primary_key = primary_key
+        self.table_alias = table_alias
 
         self.auto_increment = None
         self.rel = {}
         self.reverse_rel = {}
 
+        for key, value in kwargs.items():
+            setattr(self, key, value)
+        self._additional_keys = set(kwargs.keys())
+
     def prepared(self):
         for field in self.fields.values():
             if field.default is not None:
@@ -2195,7 +2256,7 @@ class ModelOptions(object):
 
 
 class BaseModel(type):
-    inheritable_options = ['database', 'indexes', 'order_by', 'primary_key']
+    inheritable = set(['database', 'indexes', 'order_by', 'primary_key'])
 
     def __new__(cls, name, bases, attrs):
         if not bases:
@@ -2208,7 +2269,8 @@ class BaseModel(type):
                 if not k.startswith('_'):
                     meta_options[k] = v
 
-        orig_primary_key = None
+        model_pk = getattr(meta, 'primary_key', None)
+        parent_pk = None
 
         # inherit any field descriptors by deep copying the underlying field
         # into the attrs of the new model, additionally see if the bases define
@@ -2218,45 +2280,46 @@ class BaseModel(type):
                 continue
 
             base_meta = getattr(b, '_meta')
+            if parent_pk is None:
+                parent_pk = deepcopy(base_meta.primary_key)
+            all_inheritable = cls.inheritable | base_meta._additional_keys
             for (k, v) in base_meta.__dict__.items():
-                if k in cls.inheritable_options and k not in meta_options:
+                if k in all_inheritable and k not in meta_options:
                     meta_options[k] = v
 
             for (k, v) in b.__dict__.items():
                 if isinstance(v, FieldDescriptor) and k not in attrs:
                     if not v.field.primary_key:
                         attrs[k] = deepcopy(v.field)
-                    elif not orig_primary_key:
-                        orig_primary_key = deepcopy(v.field)
 
         # initialize the new class and set the magic attributes
         cls = super(BaseModel, cls).__new__(cls, name, bases, attrs)
         cls._meta = ModelOptions(cls, **meta_options)
         cls._data = None
-
-        primary_key = None
+        cls._meta.indexes = list(cls._meta.indexes)
 
         # replace fields with field descriptors, calling the add_to_class hook
         for name, attr in list(cls.__dict__.items()):
-            cls._meta.indexes = list(cls._meta.indexes)
             if isinstance(attr, Field):
                 attr.add_to_class(cls, name)
-                if attr.primary_key:
-                    primary_key = attr
+                if attr.primary_key and model_pk:
+                    raise ValueError('primary key is overdetermined.')
+                elif attr.primary_key:
+                    model_pk = attr
 
-        if primary_key is None:
-            if orig_primary_key:
-                primary_key = orig_primary_key
-                name = primary_key.name
+        if model_pk is None:
+            if parent_pk:
+                model_pk, name = parent_pk, parent_pk.name
             else:
-                primary_key = PrimaryKeyField(primary_key=True)
-                name = 'id'
-            primary_key.add_to_class(cls, name)
+                model_pk, name = PrimaryKeyField(primary_key=True), 'id'
+            model_pk.add_to_class(cls, name)
+        elif isinstance(model_pk, CompositeKey):
+            model_pk.add_to_class(cls, '_composite_key')
 
-        cls._meta.primary_key = primary_key
+        cls._meta.primary_key = model_pk
         cls._meta.auto_increment = (
-            isinstance(primary_key, PrimaryKeyField) or
-            bool(primary_key.sequence))
+            isinstance(model_pk, PrimaryKeyField) or
+            bool(model_pk.sequence))
         if not cls._meta.db_table:
             cls._meta.db_table = re.sub('[^\w]+', '_', cls.__name__.lower())
 
@@ -2377,6 +2440,9 @@ class Model(with_metaclass(BaseModel)):
     def set_id(self, id):
         setattr(self, self._meta.primary_key.name, id)
 
+    def pk_expr(self):
+        return self._meta.primary_key == self.get_id()
+
     def prepared(self):
         pass
 
@@ -2394,20 +2460,16 @@ class Model(with_metaclass(BaseModel)):
             field_dict = self._prune_fields(field_dict, only)
         if self.get_id() is not None and not force_insert:
             field_dict.pop(pk.name, None)
-            update = self.update(
-                **field_dict
-            ).where(pk == self.get_id())
-            update.execute()
+            self.update(**field_dict).where(self.pk_expr()).execute()
         else:
-            if self._meta.auto_increment:
-                field_dict.pop(pk.name, None)
-            insert = self.insert(**field_dict)
-            new_pk = insert.execute()
-            if self._meta.auto_increment:
-                self.set_id(new_pk)
+            pk = self.get_id()
+            ret_pk = self.insert(**field_dict).execute()
+            if ret_pk is not None:
+                pk = ret_pk
+            self.set_id(pk)
 
     def dependencies(self, search_nullable=False):
-        query = self.select().where(self._meta.primary_key == self.get_id())
+        query = self.select().where(self.pk_expr())
         stack = [(type(self), query)]
         seen = set()
 
@@ -2432,8 +2494,7 @@ class Model(with_metaclass(BaseModel)):
                     model.update(**{fk.name: None}).where(query).execute()
                 else:
                     model.delete().where(query).execute()
-        return self.delete().where(
-            self._meta.primary_key == self.get_id()).execute()
+        return self.delete().where(self.pk_expr()).execute()
 
     def __eq__(self, other):
         return (
@@ -2452,7 +2513,7 @@ def prefetch_add_subquery(sq, subqueries):
             subquery = subquery.select()
         subquery_model = subquery.model_class
         fkf = None
-        for j in range(i, -1, -1):
+        for j in reversed(range(i + 1)):
             last_query = fixed_queries[j][0]
             fkf = subquery_model._meta.rel_for_model(last_query.model_class)
             if fkf:
