@@ -32,6 +32,7 @@ def get_option_parser():
     cases.add_option('--proxy', dest='proxy', default=False, action='store_true', help='proxy tests')
     cases.add_option('--pwiz', dest='pwiz', default=False, action='store_true', help='pwiz, schema introspector and model generator')
     cases.add_option('--signals', dest='signals', default=False, action='store_true', help='signals tests')
+    cases.add_option('--shortcuts', dest='shortcuts', default=False, action='store_true', help='shortcuts tests')
     cases.add_option('--sqlite-ext', dest='sqlite_ext', default=False, action='store_true', help='sqlite_ext tests')
     cases.add_option('--test-utils', dest='test_utils', default=False, action='store_true', help='test_utils tests')
 
@@ -73,6 +74,9 @@ def collect_modules(options):
     if xtra(options.signals):
         from playhouse import tests_signals
         modules.append(tests_signals)
+    if xtra(options.shortcuts):
+        from playhouse import tests_shortcuts
+        modules.append(tests_shortcuts)
     if xtra(options.sqlite_ext):
         from playhouse import tests_sqlite_ext
         modules.append(tests_sqlite_ext)
